# What is GitLab CI/CD?

GitLab CI/CD is a powerful built-in tool that allows you to automate the building, testing, and deployment of your applications. It uses a YAML file called .gitlab-ci.yml in your project's root directory to define your CI/CD pipeline.

## Key Concepts

- **Pipeline**: A series of stages that define the workflow of your CI/CD process.
- **Stage**: A logical grouping of jobs, such as build, test, and deploy.
- **Job**: A single unit of work, such as compiling code, running tests, or deploying an application.
- **Runner**: An agent that executes jobs. Runners can be shared, specific to a project, or your own.
- **Executor**: The environment in which a job runs. Examples include shell, docker, and kubernetes.

```yaml
# Simple .gitlab-ci.yml file (shell executor by default)
stages:
  - build

hello_world:
  stage: build
  script:
    - echo "Hello, World!"

```

```yaml
# Simple app .gitlab-ci.yml (shell executor by default)
stages:
  - build
  - test

build:
  stage: build
  script:
    - echo "Building the application..."
    - mkdir build
    - cd build
    - cmake ..
    - make

test:
  stage: test
  script:
    - echo "Running tests..."
    - cd build
    - ctest
```
