#!/bin/bash
PASS=$1
STUDENT=$2
PUBLIC_KEY=$3

# ----- CREATES USER GROUPS
echo "root:$PASS" | chpasswd
useradd -m -U -s /bin/bash $STUDENT
usermod -aG sudo $STUDENT
echo "$STUDENT:$PASS" | chpasswd

# Add public key to authorized_keys
mkdir -p /home/$STUDENT/.ssh
echo "$PUBLIC_KEY" >> /home/$STUDENT/.ssh/authorized_keys
chown -R $STUDENT:$STUDENT /home/$STUDENT/.ssh
chmod 600 /home/$STUDENT/.ssh/authorized_keys

# --- disable daily upgrade and update
systemctl stop apt-daily.service
systemctl stop apt-daily.timer
systemctl stop apt-daily-upgrade.timer
systemctl disable apt-daily.service
systemctl disable apt-daily.timer
systemctl disable apt-daily-upgrade.timer
systemctl kill --kill-who=all apt-daily.service

export DEBIAN_FRONTEND=noninteractive

echo "10.50.255.254 vta" > /etc/resolv.conf
echo "search vta" >>/etc/resolv.conf
echo "nameserver 10.50.255.254" >>/etc/resolv.conf

# ----- ENABLE SUDO NOPASSWD
sed -i '/# %wheel        ALL=(ALL)       NOPASSWD: ALL/ c\%wheel        ALL=(ALL)       NOPASSWD: ALL' /etc/sudoers

# ----- ENABLES SSH
cat <<EOF > /tmp/sshd_temp
Include /etc/ssh/sshd_config.d/*.conf

Port 22
PermitRootLogin no
KbdInteractiveAuthentication no
UsePAM yes
AllowTcpForwarding yes
X11Forwarding yes
PrintMotd no
AcceptEnv LANG LC_*
Subsystem    sftp    /usr/lib/openssh/sftp-server
EOF

mv /tmp/sshd_temp /etc/ssh/sshd_config && echo "SSHD CONFIG UPDATED"
service sshd restart && echo "SSHD RESTARTED"

# ----- INSTALL ADDITIONAL SOFTWARE

apt-get update
apt-get upgrade -y
apt-get install ubuntu-desktop -y

# Remove legacy docker packages
for pkg in docker.io docker-doc docker-compose docker-compose-v2 podman-docker containerd runc; do sudo apt-get remove $pkg; done


# Install docker
curl -fsSL https://get.docker.com | sh
docker pull alpine/ansible:latest
docker pull hashicorp/terraform:latest

# Install pulumi
curl -fsSL https://get.pulumi.com | sh
mv ~/.pulumi/bin/* /bin/

# Install uv (Python env/package manager)
curl -LsSf https://astral.sh/uv/install.sh | sh
mv $HOME/.local/bin/uv /bin/uv
mv $HOME/.local/bin/uvx /bin/uvx

cd /root/woac-infra-demo/uv-env
uv sync

# Add aliases to .bashrc
echo "alias terraform='docker run -it -v $(pwd):/app -w /app hashicorp/terraform:latest'" >> /root/.bashrc
# echo "alias pulumi='docker run -it -v $(pwd):/app -w /app pulumi/pulumi:latest'" >> /root/.bashrc

# Blank Horizon SPICE console fix (can remove upon transition to VNC)
echo "X-GNOME-Autostart-enabled=false" > /etc/xdg/autostart/spice-vdagent.desktop
echo "X-GNOME-Autostart-enabled=false" > /usr/share/gdm/autostart/LoginWindow/spice-vdagent.desktop
systemctl stop spice-vdagent
systemctl disable spice-vdagent
