#!/bin/bash ------------------------------------------------------------------------------------

# ------------------------------------------------------------ #
# Links
# ------------------------------------------------------------ #

    # Openstack ENV Variables
        https://docs.openstack.org/python-openstackclient/latest/cli/man/openstack.html
    # Example Heat Templates
        https://github.com/openstack/heat-templates

# ------------------------------------------------------------ #
# Keystone (Identity)
# ------------------------------------------------------------ #

# Issue an auth token
AUTH_TOKEN="$(openstack token issue -f value -c id)"
curl https://vta.cybbh.space:7480/swift/v1/AUTH_1f9100d4e6944692a52319bd09f20f36/deployment/pe/External_DMZ_Webserver/software.zip -H "X-Auth-Token: ${AUTH_TOKEN}" > /tmp/file.zip
# clean up with:
openstack token revoke ${AUTH_TOKEN}

# Add/Remove a user to a project
openstack role add --project ${PROJECT} --user ${USER_NAME} --user-domain ipa member
openstack role add --project dco --user kevin.m.crotty --user-domain ipa member
openstack role remove --project ${PROJECT} --user ${USER_NAME} --user-domain ipa member

# Add admin to a project
openstack role add --project ${PROJECT} --user admin user

# Display users assigned to a project
openstack role assignment list --user-domain ipa --project ${PROJECT} --names

# List members of an IPA/Openstack group
openstack user list --group ${GROUP}


# Create a project
openstack project create --domain default --description "Test project" test-project

# Update a project description
openstack project set --description "New description" test-project

# List all users
openstack user list

# Create a new user
openstack user create --domain default --password-prompt newuser

# Update a user's password
openstack user password set --password-prompt newuser

# Display the role assignments for a user
openstack role assignment list --user ${USER_NAME} --user-domain ipa

# Create a group
openstack group create --domain default test-group

# Add/Remove a user to/from a group
openstack group add user test-group newuser
openstack group remove user test-group newuser

# Display the role assignments for a group
openstack role assignment list --group test-group

# Add/Remove a role to/from a group
openstack role add --group test-group --project ${PROJECT} user
openstack role remove --group test-group --project ${PROJECT} user

# Create a domain
openstack domain create --description "Test Domain" test-domain

# Update a domain description
openstack domain set --description "New description" test-domain

# List roles
openstack role list

# Create a new role
openstack role create test-role

# Delete a role
openstack role delete test-role

# List services
openstack service list

# Create a new service
openstack service create --name test-service --description "Test Service" test

# Delete a service
openstack service delete test-service

# List endpoints
openstack endpoint list

# Create a new endpoint
openstack endpoint create --region RegionOne test public http://test.example.com

# Delete an endpoint
openstack endpoint delete test

# List regions
openstack region list

# Create a new region
openstack region create test-region

# Delete a region
openstack region delete test-region


# ------------------------------------------------------------ #
# Neutron (Networking)
# ------------------------------------------------------------ #

# --- Networks --- #

# Show details for a specific network
openstack network show ${NETWORK_ID}

# List all agents associated with networks
openstack network agent list

# Show details for a specific network
openstack network show ${NETWORK_ID}

# Create a new network
openstack network create ${NETWORK_NAME}

# --- Ports --- #

# Display ports on a specific instance
openstack port list --server ${INSTANCE_ID}

# Show details for a specific port
openstack port show ${PORT_ID}

# Show details for a specific port
openstack port show $PORT_ID

# List DHCP ports on a specific network
openstack port list --device-owner network:dhcp --network ${NETWORK_ID}

# List router ports in a project
openstack port list --project ${PROJECT_ID} --device-owner network:router

# List router interface ports in a project
openstack port list --project ${PROJECT_ID} --device-owner network:router_interface

# List binding host IDs of router interface ports in a project
openstack port list --project ${PROJECT_ID} --device-owner network:router_interface -f binding_host_id

# List binding host IDs of router interface ports in a project (compact output)
openstack port list --project ${PROJECT_ID} --device-owner network:router_interface -c binding_host_id

# List binding host IDs of router interface ports in a project (compact output)
openstack port list --project ${PROJECT_ID} --device-owner network:router_interface -c binding_host_id

# Display ports on a specific instance
openstack port list --server ${INSTANCE_ID}

# Show details for a specific port
openstack port show ${PORT_ID}

# List DHCP ports on a specific network
openstack port list --device-owner network:dhcp --network ${NETWORK_ID}

# Create a new port
openstack port create --network ${NETWORK_ID}

# Attach a port to an instance
openstack server add port ${INSTANCE_ID} ${PORT_ID}

# Detach a port from an instance
openstack server remove port ${INSTANCE_ID} ${PORT_ID}

# Delete a port
openstack port delete ${PORT_ID}

# --- Subnets --- #

# Show details for a specific subnet
openstack subnet show ${SUBNET_ID}

# Show details for a specific subnet
openstack subnet show ${SUBNET_ID}

# Create a new subnet
openstack subnet create --network ${NETWORK_ID} --subnet-range ${CIDR} ${SUBNET_NAME}

# --- Routers --- #
# Create a new router

openstack router create ${ROUTER_NAME}

# Set the gateway for a router
openstack router set --external-gateway ${EXTERNAL_NETWORK_ID} ${ROUTER_ID}

# Add an interface to a router
openstack router add subnet ${ROUTER_ID} ${SUBNET_ID}

# Remove an interface from a router
openstack router remove subnet ${ROUTER_ID} ${SUBNET_ID}

# --- Floating IPs --- #

# Create a new floating IP
openstack floating ip create ${EXTERNAL_NETWORK_ID}

# Associate a floating IP with a port
openstack floating ip set --port ${PORT_ID} ${FLOATING_IP_ID}

# Disassociate a floating IP from a port
openstack floating ip unset --port ${FLOATING_IP_ID}

# Delete a floating IP
openstack floating ip delete ${FLOATING_IP_ID}


# --- Security Groups --- #

# Create a new security group
openstack security group create ${SECURITY_GROUP_NAME} --description "${DESCRIPTION}"

# Add a rule to allow incoming SSH traffic to a security group
openstack security group rule create --proto tcp --dst-port 22 ${SECURITY_GROUP_NAME}

# Add a rule to allow incoming ICMP traffic to a security group
openstack security group rule create --proto icmp ${SECURITY_GROUP_NAME}

# Add a rule to allow incoming HTTP traffic to a security group
openstack security group rule create --proto tcp --dst-port 80 ${SECURITY_GROUP_NAME}

# Add a rule to allow incoming HTTPS traffic to a security group
openstack security group rule create --proto tcp --dst-port 443 ${SECURITY_GROUP_NAME}

# Delete a security group rule
openstack security group rule delete ${RULE_ID}

# List all available keypairs
openstack keypair list

# Create a new keypair
openstack keypair create ${KEYPAIR_NAME} > ${KEYPAIR_NAME}.pem

# ------------------------------------------------------------ #
# Nova (Compute)
# ------------------------------------------------------------ #

# --- Projects --- #

# Change or set project quotas
openstack quota set --ram 81920 --instances 20 --cores 80 --volumes 20 ${PROJECT}
openstack quota set --force --floating-ips 300 --gigabytes 16384 --instances 2000 \
    --ram 6144000 --cores 2000 --networks 1200 --ports 3500 --routers 160 \
    --secgroup-rules 600 --secgroups 120 --subnets 1200 --volumes 60 ${PROJECT}
openstack quota set --force --floating-ips 100 --gigabytes 16384 --instances 1000 --ram 2624400 --cores 1600 --networks 400 --ports 1800 --routers 60 --secgroup-rules 240 --secgroups 40 sec-6-xl
# --- Instances --- #

# Show console log from an instance
openstack console log show ${INSTANCE_ID}

# Get the spice console url for an instance
openstack console url show ${INSTANCE_ID} --spice

# --- Flavors --- #

# Create flavor examples
openstack flavor create --ram 6144 --disk 64 --vcpus 4 --public --property hw:cpu_sockets=2 --property hw:cpu_cores=2 cy.win_64
openstack flavor create --ram 32768 --disk 32 --ephemeral 0 --vcpus 12 --private --project ${PROJECT_ID} mem.es3
openstack flavor create --ram 16384 --disk 64 --vcpus 12 --public mem.es3

# Boot a new instance with the specified image, flavor, keypair and security group
openstack server create --image ${IMAGE_ID} --flavor ${FLAVOR_ID} --key-name ${KEYPAIR_NAME} --security-group ${SECURITY_GROUP_NAME} ${INSTANCE_NAME}

# Associate a floating IP with an instance
openstack server add floating ip ${INSTANCE_ID} ${FLOATING_IP}

# Reboot an instance
openstack server reboot ${INSTANCE_ID}

# Delete an instance
openstack server delete ${INSTANCE_ID}

# ------------------------------------------------------------ #
# Cinder (Block Storage)
# ------------------------------------------------------------ #

# Create a new volume
openstack volume create --size ${SIZE_IN_GB} ${VOLUME_NAME}

# Attach a volume to an instance
openstack server add volume ${INSTANCE_ID} ${VOLUME_ID}

# Detach a volume from an instance
openstack server remove volume ${INSTANCE_ID} ${VOLUME_ID}

# Delete a volume
openstack volume delete ${VOLUME_ID}

# Create a snapshot of an instance
openstack server image create --name ${SNAPSHOT_NAME} ${INSTANCE_ID}

# List all available images
openstack image list

# Add a project to an image
openstack image add project ${IMAGE_ID} ${PROJECT_ID}
openstack image set --accept --project ${PROJECT_ID} ${IMAGE_ID}


# ------------------------------------------------------------ #
# Heat (Orchestration)
# ------------------------------------------------------------ #

# Check the template used to create a stack
openstack stack template show ${STACK_NAME}

# Create a stack from yaml file
openstack stack create ${STACK_NAME} --template ${YAML}

# Update a stack with a new yaml file
openstack stack update ${STACK_NAME} --template ${NEW_YAML}

# Preview the changes that will be made to a stack
openstack stack preview update ${STACK_NAME} --template ${NEW_YAML}

# Rollback a stack update
openstack stack snapshot create ${STACK_NAME} --name "snapshot-1"
openstack stack update ${STACK_NAME} --rollback --snapshot "snapshot-1"

# Save and restore stack details
openstack stack snapshot create ${STACK_NAME} --name "snapshot-1"
openstack stack snapshot show ${STACK_NAME} "snapshot-1"
openstack stack snapshot restore ${STACK_NAME} "snapshot-1"

# Check the dependencies of a stack
openstack stack dependency list ${STACK_NAME}

# Check the resources for a specific stack
openstack stack resource show ${STACK_NAME} ${RESOURCE_NAME}

# Check the details of a specific stack output
openstack stack output show ${STACK_NAME} ${OUTPUT_NAME}

# Display all the resources for a stack
openstack stack resource list ${STACK_ID}

# Display all the outputs of a stack
openstack stack output show --all ${STACK_ID}

# Suspend a stack
openstack stack suspend ${STACK_NAME}

# Resume a suspended stack
openstack stack resume ${STACK_NAME}

# Check the status of a stack
openstack stack show ${STACK_NAME}

# Delete a stack
openstack stack delete ${STACK_NAME}

# Check the events of a stack
openstack stack event list ${STACK_NAME}

# --- Cloud Init --- #
# Look at Instance Cloud Init Data
curl 169.254.169.254/latest/user-data

# Remove cloud-init artifacts
/media/root/config-2/ec2/latest/user-data
rm -rf /var/lib/cloud/instance/*
chattr +i /var/lib/cloud/instance/

# ------------------------------------------------------------ #
# Misc
# ------------------------------------------------------------ #

# Example of selecting a specific version of the compute API
openstack --os-compute-api-version 2.74 server create --flavor vgpu_1 --image ${IMGAGE_ID} --network ${NETWORK_ID} --host ${HOST_ID} --wait test_vgpu

# Example of selecting ID column and feeding results to xargs to delete each resource
openstack ${RESOURCE_TYPE} list -f value -c ID | xargs -n1 openstack ${RESOURCE_TYPE} delete

# Example turning off the api limit cap
openstack server list --all-projects --limit -1 | grep ${IP}

# Delete an docker container resource (Zun)
openstack appcontainer delete --force ${APP_CONTAINER_ID}

# Run openstack commands from inside a container containing the openstack client
podman run --userns keep-id --name test -it --rm -v /home/dayman/:/home/dayman -e OS_CLOUD=$OS_CLOUD kmc-env:310 /bin/bash

#!/bin/bash ------------------------------------------------------------------------------------