# Provider configuration
terraform {
  required_providers {
    openstack = {
      source = "terraform-provider-openstack/openstack"
    }
  }
}

# Data
data "openstack_networking_network_v2" "public" {
  name = "public"
}

# Variables
variable "username" {
  type        = string
  description = "Sets the login username for the instances"
  default     = "student"
}

variable "password" {
  type        = string
  description = "Sets the Login Password for the instances"
  default     = "password"
  sensitive   = true
}

variable "public_key" {
  type        = string
  description = "Sets the public key for the instances"
  default     = "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIIwofrs+eeWoyxUZH89SgPsZbxn2sgpUOTd7n3LqoDdi woac-infra-demo"
}

# Random string generation
resource "random_string" "suffix" {
  length  = 4
  special = false
}

# Network Configuration
resource "openstack_networking_network_v2" "ops_network_test" {
  name           = "ops_network_test_${random_string.suffix.result}"
  admin_state_up = true
}

resource "openstack_networking_subnet_v2" "ops_subnet_test" {
  name            = "ops_subnet_test"
  network_id      = openstack_networking_network_v2.ops_network_test.id
  cidr            = "192.168.65.0/27"
  gateway_ip      = "192.168.65.30"
  dns_nameservers = ["10.50.255.254"]
  ip_version      = 4
}

# Router Configuration
resource "openstack_networking_router_v2" "ops_router_test" {
  name                = "ops_router_test"
  external_network_id = data.openstack_networking_network_v2.public.id
}

resource "openstack_networking_router_interface_v2" "ops_router_interface_test" {
  router_id = openstack_networking_router_v2.ops_router_test.id
  subnet_id = openstack_networking_subnet_v2.ops_subnet_test.id
}

# Linux Analyst Workstation Port
resource "openstack_networking_port_v2" "linux_opstation_port_test" {
  name                  = "linux_opstation_port_test"
  network_id            = openstack_networking_network_v2.ops_network_test.id
  port_security_enabled = false

  fixed_ip {
    subnet_id  = openstack_networking_subnet_v2.ops_subnet_test.id
    ip_address = "192.168.65.20"
  }
}

# Floating IP Configuration
resource "openstack_networking_floatingip_v2" "linux_opstation_float_ip_test" {
  pool = "public"
}

resource "openstack_networking_floatingip_associate_v2" "linux_opstation_float_ip_assoc_test" {
  floating_ip = openstack_networking_floatingip_v2.linux_opstation_float_ip_test.address
  port_id     = openstack_networking_port_v2.linux_opstation_port_test.id
}

# Linux Analyst Workstation Instance
resource "openstack_compute_instance_v2" "linux_opstation_test" {
  name            = "linux_opstation_test"
  image_name      = "delta_ops"
  flavor_name     = "m4.medium.2"
  config_drive    = true
  user_data       = templatefile("${path.module}/ud.sh", {
    username   = var.username
    password   = var.password
    public_key = var.public_key
  })

  network {
    port = openstack_networking_port_v2.linux_opstation_port_test.id
  }
}